
setMethod("gateNodeToFilterObject",
          signature(aNode = "XMLInternalElementNode"), 
          definition = function(aNode, 
                                quiet=TRUE, 
                                stripComp=TRUE,
                                version="7.2.2") {

    # Older versions of FlowJo had scatter gate coordinates at 1/64
    # of their "actual" value.   See further below.
    multiplier = 64;
    if (compareVersion(version, "7.2.2") > 0) { multiplier = 1; }  
    gateType   = xmlName(aNode);

    knownGateTypes = c("PolygonGate", "RectangleGate", "RangeGate",
        "EllipsoidGate")
    if (!(gateType %in% knownGateTypes)) {
        if (quiet == FALSE) {
            print(paste("Sorry, I don't know what a '", gateType,
                "' gate is...", sep = ""))
        }
        return(NULL)
    }

    ancs     = xmlAncestors(aNode);
    popNode  = ancs[length(ancs)-1];
    gateName = xmlGetAttr(popNode[[1]], "name");

    if (gateType == "PolygonGate") {
        axesNames = unlist(xpathApply(aNode, ".//Axis", xmlGetAttr, "name"));
        if (stripComp == TRUE) {
            axesNames = gsub("\\bComp-", "", axesNames, perl = TRUE)
        }

        xPoints = as.numeric(unlist(xpathApply(aNode, ".//Point", xmlGetAttr, "x")));
        yPoints = as.numeric(unlist(xpathApply(aNode, ".//Point", xmlGetAttr, "y")));
        verticies = matrix(c(xPoints, yPoints), ncol = 2, nrow = length(xPoints),
            byrow = FALSE);
        colnames(verticies) = axesNames;

        # Vexingly, FlowJo puts their scatter gate coordinates in at 1/64 
        # of the correct values.  Never quite figured this out.
        for (i in 1:2) {
            if (length(grep("FSC-[AHW]|SSC-[AHW]", axesNames[i])) > 0) {
                verticies[, i] = multiplier * verticies[, i]
            }
        }
        thisGate = polygonGate(filterId = gateName, .gate = verticies)
        thisGate@filterId = gateName
        return(thisGate)
    }
    if ((gateType == "RectangleGate") | (gateType == "RangeGate")) {
      if (xmlSize(aNode) == 1) {
        rangeNames = unlist(xpathApply(aNode, ".//Axis", xmlGetAttr, "name"));
        rangeMins  = as.numeric(xmlGetAttr(aNode, "min"));
        rangeMaxes = as.numeric(xmlGetAttr(aNode, "max"));
      } else {
        rangeNames = unlist(xpathApply(aNode, ".//RangeGate/Axis", xmlGetAttr, "name"));
        rangeMins  = as.numeric(unlist(xpathApply(aNode, ".//RangeGate", xmlGetAttr, "min")));
        rangeMaxes = as.numeric(unlist(xpathApply(aNode, ".//RangeGate", xmlGetAttr, "max")));
      }
      
      if (stripComp == TRUE) {
        rangeNames = gsub("\\bComp-", "", rangeNames, perl = TRUE)
      }
      
      verticies = matrix(c(rangeMins, rangeMaxes), ncol = length(rangeNames),
          nrow = 2, byrow = TRUE);
      colnames(verticies) = rangeNames;
      
      # Vexingly, FlowJo puts their scatter gate coordinates in at 1/64 
      # of the correct values.  Never quite figured this out.
      for (i in 1:length(rangeNames)) {
          if (length(grep("FSC-[AHW]|SSC-[AHW]", rangeNames[i])) > 0) {
              verticies[, i] = multiplier * verticies[, i]
          }
      }
      thisGate = rectangleGate(filterId = gateName, .gate=verticies);
      thisGate@filterId = gateName
      return(thisGate)
    }
    if (gateType == "EllipseGate") {
        if (quiet == FALSE) {
            print("Sorry, I haven't worked out EllipseGates yet.")
        }
        return(NULL)
    }
});



