setMethod("readFlowJoFile",
          signature(xmlFile = "character"), 
          definition = function(xmlFile) {
  
    ## Check whether the FlowJo workspace exists.  Not sure how to 
    ## check if it is valid XML though.
    if (! file.exists(xmlFile)) {
      stop(paste("I don't see a file at:", xmlFile)); 
    } else {
  
      ## Initial load of XML data file into in-RAM DOM tree
      flowJoDOM <- xmlTreeParse(xmlFile, useInternalNodes=TRUE)
    
      ## Get the root of the DOM tree; expect it to be a "Workspace" node
      root <- xmlRoot(flowJoDOM);
      if(xmlName(root) != "Workspace") {
        stop("Top node is not a 'Workspace' node.  Is this really a FlowJo workspace?!?");
      }
    
      ## This Workspace should refer to at least one or more FCS files
      numSamples = length(xpathApply(flowJoDOM,
                                     path="/Workspace/SampleList/Sample/DataSet",
                                     xmlChildren));
                                     
      if (numSamples < 1) {
        warning("This workspace doesn't refer to any FCS files!");
      }


      numGates = length(xpathApply(flowJoDOM,
                                   path="/Workspace/SampleList/Sample/SampleNode/Subpopulations",
                                   xmlChildren));
                                     
      if (numGates < 1) {
        warning("This workspace doesn't contain any gates!");
      }


      return(flowJoDOM);
    }            
});




